package de.intevation.flys.artifacts;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.state.DefaultOutput;
import de.intevation.artifactdatabase.state.State;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.ArtifactFactory;
import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.CallMeta;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WQKms;
import de.intevation.flys.artifacts.model.WQKmsFacet;
import de.intevation.flys.artifacts.model.WKmsFactory;
import de.intevation.flys.artifacts.model.WQKmsFactory;

import de.intevation.flys.artifacts.states.StaticState;
import de.intevation.flys.artifacts.resources.Resources;

import de.intevation.artifacts.common.utils.XMLUtils;

/**
 * Artifact to access additional "waterlevel/discharge"-type of data, like
 * fixation measurements.
 *
 * This artifact neglects (Static)FLYSArtifacts capabilities of interaction
 * with the StateEngine by overriding the getState*-methods.
 */
public class StaticWQKmsArtifact
extends      StaticFLYSArtifact
implements   FacetTypes
{
    /** The logger for this class. */
    private static Logger logger =
        Logger.getLogger(StaticWQKmsArtifact.class);

    /** XPath to access initial parameter. */
    public static final String XPATH_DATA =
        "/art:action/art:ids/@value";

    public static final String STATIC_STATE_NAME =
        "state.additional_wqkms.static";

    /** One and only state to be in. */
    protected transient State state = null;


    /**
     * Trivial Constructor.
     */
    public StaticWQKmsArtifact() {
        logger.debug("StaticWQKmsArtifact.StaticWQKmsArtifact");
    }


    /**
     * Gets called from factory, to set things up.
     */
    @Override
    public void setup(
        String          identifier,
        ArtifactFactory factory,
        Object          context,
        CallMeta        callMeta,
        Document        data)
    {
        logger.debug("StaticWQKmsArtifact.setup");

        state = new StaticState(STATIC_STATE_NAME);

        List<Facet> fs = new ArrayList<Facet>();
        logger.debug(XMLUtils.toString(data));
        String code = XMLUtils.xpathString(
            data, XPATH_DATA, ArtifactNamespaceContext.INSTANCE);

        // TODO Go for JSON, one day.
        //ex.: flood_protection-wstv-114-12
        if (code != null) {
            String [] parts = code.split("-");

            if (parts.length >= 4) {
                try {
                    int col = Integer.valueOf(parts[2]);
                    int wst = Integer.valueOf(parts[3]);

                    addStringData("col_pos", parts[2]);
                    addStringData("wst_id",  parts[3]);

                    String wkmsName = WKmsFactory.getWKmsName(col, wst);

                    String name;
                    if (parts[0].equals(HEIGHTMARKS_POINTS)) {
                        name = HEIGHTMARKS_POINTS;
                    }
                    else {
                        name = STATIC_WQKMS;
                    }

                    Facet facet = new WQKmsFacet(
                        name,
                        Resources.getMsg(
                            callMeta,
                            wkmsName,
                            wkmsName));
                    fs.add(facet);
                    facets.put(state.getID(), fs);
                }
                catch (Exception e) {}
            }
        }

        spawnState();
        super.setup(identifier, factory, context, callMeta, data);
    }


    /**
     * Initialize the static state with output.
     * @return static state
     */
    protected State spawnState() {
        state = new StaticState(STATIC_STATE_NAME);
        List<Facet> fs = facets.get(STATIC_STATE_NAME);
        DefaultOutput output1 = new DefaultOutput(
            "w_differences",
            "w_differences.longitudinal_section", "image/png",
            fs,
            "chart");
        DefaultOutput output2 = new DefaultOutput(
            "longitudinal_section",
            "longitudinal_section.longitudinal_section", "image/png",
            fs,
            "chart");

        state.getOutputs().add(output1);
        state.getOutputs().add(output2);
        return state;
    }


    /**
     * Called via setup.
     *
     * @param artifact The master-artifact.
     */
    @Override
    protected void initialize(
        Artifact artifact,
        Object context,
        CallMeta meta)
    {
        logger.debug("StaticWQKmsArtifact.initialize");
        WINFOArtifact winfo = (WINFOArtifact) artifact;
        // TODO: The river is of no interest, so far.
        addData("river", winfo.getData("river"));
    }


    /**
     * Get a list containing the one and only State.
     * @param  context ignored.
     * @return list with one and only state.
     */
    @Override
    protected List<State> getStates(Object context) {
        ArrayList<State> states = new ArrayList<State>();
        states.add(getState());
        return states;
    }


    /**
     * Get the "current" state (there is but one).
     * @param cc ignored.
     * @return the "current" (only possible) state.
     */
    @Override
    public State getCurrentState(Object cc) {
        return getState();
    }


    /**
     * Get the only possible state.
     * @return the state.
     */
    protected State getState() {
        return getState(null, null);
    }


    /**
     * Get the state.
     * @param context ignored.
     * @param stateID ignored.
     * @return the state.
     */
    @Override
    protected State getState(Object context, String stateID) {
        return (state != null)
            ? state
            : spawnState();
    }


    /**
     * Get WQKms from factory.
     * @param TODO idx param is not needed
     * @return WQKms according to parameterization (can be null);
     */
    public WQKms getWQKms(int idx) {
        logger.debug("StaticWQKmsArtifact.getWQKms");

        return WQKmsFactory.getWQKms(
            Integer.valueOf(getDataAsString("col_pos")),
            Integer.valueOf(getDataAsString("wst_id")));
    }


    /**
     * Determines Facets initial disposition regarding activity (think of
     * selection in Client ThemeList GUI). This will be checked one time
     * when the facet enters a collections describe document.
     *
     * @param facetName name of the facet.
     * @param index     index of the facet.
     *
     * @return Always 0. Static Data will enter plots inactive.
     */
    @Override
    public int getInitialFacetActivity(
        String outputName,
        String facetName,
        int index)
    {
        return 0;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
